@@ -368,6 +368,7 @@ typedef struct {
     struct lp_style_type box;	/* linetype of box around key:  */
     char *font;			/* Will be used for both key title and plot titles */
     struct t_colorspec textcolor;	/* Will be used for both key title and plot titles */
+    struct t_colorspec fillcolor;	/* only used if "set key front" */
     BoundingBox bounds;
     int maxcols;		/* maximum no of columns for horizontal keys */
     int maxrows;		/* maximum no of rows for vertical keys */
@@ -391,6 +392,7 @@ extern legend_key keyT;
 		FALSE, FALSE, FALSE, TRUE, \
 		DEFAULT_KEYBOX_LP, \
 		NULL, {TC_LT, LT_BLACK, 0.0}, \
+		BACKGROUND_COLORSPEC, \
 		{0,0,0,0}, 0, 0, \
 		EMPTY_LABELSTRUCT}
 
@@ -557,7 +559,7 @@ extern int current_x11_windowid;
 #define LAYER_BEHIND     -1
 #define LAYER_BACK        0
 #define LAYER_FRONT       1
-#define LAYER_FOREGROUND  2	/* not currently used */
+#define LAYER_FOREGROUND  2	/* used only for axis tic labels */
 #define LAYER_FRONTBACK   4	/* used only by grid walls */
 #define LAYER_DEPTHORDER  8	/* for objects to be included in pm3d depth sorting */
 #define LAYER_PLOT	 16	/* currently used only by fig.trm */
