@@ -597,6 +597,10 @@ parse_primary_expression()
 		at_highest_column_used = a.v.int_val;
 	    add_action(DOLLARS)->v_arg = a;
 	}
+    } else if (equals(c_token, "$#")) {
+	struct value a = {INTGR, {DOLLAR_NCOLUMNS}};
+	c_token++;
+	add_action(DOLLARS)->v_arg = a;
     } else if (equals(c_token, "|")) {
 	struct udvt_entry *udv;
 	c_token++;
@@ -650,6 +654,10 @@ parse_primary_expression()
 		if (!strcmp(ft[whichfunc].f_name,"timecolumn"))
 		    add_action(PUSHC)->v_arg = num_params;
 
+		/* These functions have an optional 3rd parameter */
+		if (!strncmp(ft[whichfunc].f_name,"weekdate_",9))
+		    add_action(PUSHC)->v_arg = num_params;
+
 		/* The column() function has side effects requiring special handling */
 		if (!strcmp(ft[whichfunc].f_name,"column")) {
 		    set_up_columnheader_parsing( &(at->actions[at->a_count-1]) );
@@ -1113,15 +1121,11 @@ parse_link_via( struct udft_entry *udf )
 static void
 parse_sum_expression()
 {
-    /* sum [<var>=<range>] <expr>
-     * - Pass a udf to f_sum (with action code (for <expr>) that is not added
-     *   to the global action table).
-     * - f_sum uses a newly created udv (<var>) to pass the current value of
-     *   <var> to <expr> (resp. its ac).
-     * - The original idea was to treat <expr> as function f(<var>), but there
-     *   was the following problem: Consider 'g(x) = sum [k=1:4] f(k)'. There
-     *   are two dummy variables 'x' and 'k' from different functions 'g' and
-     *   'f' which would require changing the parsing of dummy variables.
+    /* sum [<var>=<start>:<end>] <expr>
+     * - <var> pushed to stack *by name*
+     * - <start> and <end> expressions pushed to stack
+     * - A new action table for <expr> is created and passed to f_sum(arg)
+     *   via arg->udf_arg
      */
 
     char *errormsg = "Expecting 'sum [<var> = <start>:<end>] <expression>'\n";
@@ -1139,10 +1143,7 @@ parse_sum_expression()
     /* <var> */
     if (!isletter(c_token))
 	int_error(c_token, errormsg);
-    /* create a user defined variable and pass it to f_sum via PUSHC, since the
-     * argument of f_sum is already used by the udf */
     m_capture(&varname, c_token, c_token);
-    add_udv(c_token);
     arg = add_action(PUSHC);
     Gstring(&(arg->v_arg), varname);
     c_token++;
@@ -1165,8 +1166,9 @@ parse_sum_expression()
 	int_error(c_token, errormsg);
     c_token++;
 
-    /* parse <expr> and convert it to a new action table. */
-    /* modeled on code from temp_at(). */
+    /* parse <expr> and convert it to a new action table.
+     * modeled on code from temp_at().
+     */
     /* 1. save environment to restart parsing */
     save_at = at;
     save_at_size = at_size;
