@@ -1333,8 +1333,10 @@ get_3ddata(struct surface_points *this_plot)
 	    /* At this point we have stored the point coordinates. Now we need to copy */
 	    /* x,y,z into the text_label structure and add the actual text string.     */
 	    if (this_plot->plot_style == LABELPOINTS) {
-		/* Aug 2018: only store INRANGE labels. This is a CHANGE. */
-		if (cp->type == INRANGE)
+		if (draw_contour && !this_plot->opt_out_of_contours)
+		    /* Contour labels are calculated and added later */
+		    ;
+		else if (cp->type == INRANGE)
 		    store_label(this_plot->labels, cp, xdatum, df_tokens[3], color);
 	    }
 
@@ -2252,10 +2254,6 @@ eval_3dplots()
 			}
 		    }
 
-		    if (this_plot != first_dataset)
-			/* copy (explicit) "with pm3d at ..." option from the first dataset in the file */
-			strcpy(this_plot->pm3d_where, first_dataset->pm3d_where);
-
 		    /* okay, we have read a surface */
 		    plot_num++;
 		    tp_3d_ptr = &(this_plot->next_sp);
@@ -2291,6 +2289,7 @@ eval_3dplots()
 			*(this_plot->labels) = *(first_dataset->labels);
 			this_plot->labels->next = NULL;
 		    }
+		    strcpy(this_plot->pm3d_where, first_dataset->pm3d_where);
 		} while (df_return != DF_EOF);
 
 		df_close();
