@@ -51,12 +51,13 @@
 #include "pm3d.h"
 #include "getcolor.h"
 
-static void save_functions__sub __PROTO((FILE *));
-static void save_variables__sub __PROTO((FILE *));
-static void save_tics __PROTO((FILE *, struct axis *));
-static void save_mtics __PROTO((FILE *, struct axis *));
-static void save_zeroaxis __PROTO((FILE *,AXIS_INDEX));
-static void save_set_all __PROTO((FILE *));
+static void save_functions__sub(FILE *);
+static void save_variables__sub(FILE *);
+static void save_tics(FILE *, struct axis *);
+static void save_mtics(FILE *, struct axis *);
+static void save_zeroaxis(FILE *,AXIS_INDEX);
+static void save_set_all(FILE *);
+static void save_justification(int just, FILE *fp);
 
 const char *coord_msg[] = {"first ", "second ", "graph ", "screen ", "character ", "polar "};
 /*
@@ -136,9 +137,9 @@ save_variables__sub(FILE *fp)
 	if (udv->udv_value.type != NOTDEFINED) {
 	    if ((udv->udv_value.type == ARRAY)
 		&& strncmp(udv->udv_name,"ARGV",4)) {
-		    fprintf(fp,"array %s[%d] = ", udv->udv_name,
-			udv->udv_value.v.value_array[0].v.int_val);
-		    save_array_content(fp, udv->udv_value.v.value_array);
+		fprintf(fp,"array %s[%d] = ", udv->udv_name,
+			(int)(udv->udv_value.v.value_array[0].v.int_val));
+		save_array_content(fp, udv->udv_value.v.value_array);
 	    } else if (strncmp(udv->udv_name,"GPVAL_",6)
 		 && strncmp(udv->udv_name,"GPFUN_",6)
 		 && strncmp(udv->udv_name,"MOUSE_",6)
@@ -196,6 +197,35 @@ save_term(FILE *fp)
 	fputs("#    EOF\n", fp);
 }
 
+/* helper function */
+void
+save_axis_label_or_title(FILE *fp, char *name, char *suffix,
+			struct text_label *label, TBOOLEAN savejust)
+{
+    fprintf(fp, "set %s%s \"%s\" ",
+	    name, suffix, label->text ? conv_text(label->text) : "");
+    fprintf(fp, "\nset %s%s ", name, suffix);
+    save_position(fp, &(label->offset), 3, TRUE);
+    fprintf(fp, " font \"%s\"", label->font ? conv_text(label->font) : "");
+    save_textcolor(fp, &(label->textcolor));
+    if (savejust && (label->pos != CENTRE)) save_justification(label->pos,fp);
+    if (label->tag == ROTATE_IN_3D_LABEL_TAG)
+	fprintf(fp, " rotate parallel");
+    else if (label->rotate == TEXT_VERTICAL)
+	fprintf(fp, " rotate");
+    else if (label->rotate)
+	fprintf(fp, " rotate by %d", label->rotate);
+    else
+	fprintf(fp, " norotate");
+
+    if (label == &title && label->boxed) {
+	fprintf(fp," boxed ");
+	if (label->boxed > 0)
+	    fprintf(fp,"bs %d ",label->boxed);
+    }
+    fprintf(fp, "%s\n", (label->noenhanced) ? " noenhanced" : "");
+}
+
 static void
 save_justification(int just, FILE *fp)
 {
@@ -238,10 +268,12 @@ save_set_all(FILE *fp)
     fprintf(fp, "\
 %sset clip points\n\
 %sset clip one\n\
-%sset clip two\n",
+%sset clip two\n\
+%sset clip radial\n",
 	    (clip_points) ? "" : "un",
 	    (clip_lines1) ? "" : "un",
-	    (clip_lines2) ? "" : "un"
+	    (clip_lines2) ? "" : "un",
+	    (clip_radial) ? "" : "un"
 	    );
 
     save_bars(fp);
@@ -269,16 +301,15 @@ save_set_all(FILE *fp)
     else
 	fprintf(fp, "set boxwidth %g %s\n", boxwidth,
 		(boxwidth_is_absolute) ? "absolute" : "relative");
+    fprintf(fp, "set boxdepth %g\n", boxdepth > 0 ? boxdepth : 0.0);
 
     fprintf(fp, "set style fill ");
     save_fillstyle(fp, &default_fillstyle);
 
-#ifdef EAM_OBJECTS
     /* Default rectangle style */
     fprintf(fp, "set style rectangle %s fc ",
 	    default_rectangle.layer > 0 ? "front" :
 	    default_rectangle.layer < 0 ? "behind" : "back");
-    /* FIXME: broke with removal of use_palette? */
     save_pm3dcolor(fp, &default_rectangle.lp_properties.pm3d_color);
     fprintf(fp, " fillstyle ");
     save_fillstyle(fp, &default_rectangle.fillstyle);
@@ -304,7 +335,6 @@ save_set_all(FILE *fp)
 	    fputs("yy\n", fp);
 	    break;
     }
-#endif
 
     if (dgrid3d) {
       if (dgrid3d_mode == DGRID3D_QNORM) {
@@ -396,11 +426,15 @@ save_set_all(FILE *fp)
     /* Save parallel axis state */
     save_style_parallel(fp);
 
-    fprintf(fp, "set key title \"%s\"", conv_text(key->title.text));
-    if (key->title.font)
-	fprintf(fp, " font \"%s\" ", key->title.font);
-    save_justification(key->title.pos, fp);
-    fputs("\n", fp);
+    if (key->title.text == NULL)
+	fprintf(fp, "set key notitle\n");
+    else {
+	fprintf(fp, "set key title \"%s\"", conv_text(key->title.text));
+	if (key->title.font)
+	    fprintf(fp, " font \"%s\" ", key->title.font);
+	save_justification(key->title.pos, fp);
+	fputs("\n", fp);
+    }
 
     fputs("set key ", fp);
     switch (key->region) {
@@ -475,7 +509,16 @@ save_set_all(FILE *fp)
 		key->swidth, key->vert_factor, key->width_fix, key->height_fix);
     fprintf(fp, "\nset key maxcolumns %d maxrows %d",key->maxcols,key->maxrows);
     fputc('\n', fp);
-    fprintf(fp, "set key %sopaque\n", key->front ? "" : "no");
+    if (key->front) {
+	fprintf(fp, "set key opaque");
+	if (key->fillcolor.lt != LT_BACKGROUND) {
+	    fprintf(fp, " lc ");
+	    save_pm3dcolor(fp, &key->fillcolor);
+	}
+	fprintf(fp, "\n");
+    } else {
+	fprintf(fp, "set key noopaque\n");
+    }
 
     if (!(key->visible))
 	fputs("unset key\n", fp);
@@ -508,10 +551,11 @@ save_set_all(FILE *fp)
 	    save_linetype(fp, &(this_label->lp_properties), TRUE);
 	}
 	save_position(fp, &this_label->offset, 3, TRUE);
-#ifdef EAM_BOXED_TEXT
-	if (this_label->boxed)
+	if (this_label->boxed) {
 	    fprintf(fp," boxed ");
-#endif
+	    if (this_label->boxed > 0)
+		fprintf(fp,"bs %d ",this_label->boxed);
+	}
 	fputc('\n', fp);
     }
     fputs("unset arrow\n", fp);
@@ -548,9 +592,10 @@ save_set_all(FILE *fp)
 	}
 	fprintf(fp, "\n");
     }
-#if TRUE || defined(BACKWARDS_COMPATIBLE)
-    fprintf(fp, "set style increment %s\n", prefer_line_styles ? "userstyles" : "default");
-#endif
+
+    /* Mostly for backwards compatibility */
+    if (prefer_line_styles)
+	fprintf(fp, "set style increment userstyles\n");
     fputs("unset style line\n", fp);
     for (this_linestyle = first_linestyle; this_linestyle != NULL;
 	 this_linestyle = this_linestyle->next) {
@@ -558,7 +603,7 @@ save_set_all(FILE *fp)
 	save_linetype(fp, &(this_linestyle->lp_properties), TRUE);
 	fprintf(fp, "\n");
     }
-	/* TODO save "set linetype" as well, or instead */ 
+
     fputs("unset style arrow\n", fp);
     for (this_arrowstyle = first_arrowstyle; this_arrowstyle != NULL;
 	 this_arrowstyle = this_arrowstyle->next) {
@@ -588,19 +633,17 @@ save_set_all(FILE *fp)
     fprintf(fp, "set style histogram ");
     save_histogram_opts(fp);
 
-#ifdef EAM_OBJECTS
+    save_pixmaps(fp);
+
     fprintf(fp, "unset object\n");
     save_object(fp, 0);
-#endif
+    fprintf(fp, "unset walls\n");
+    save_walls(fp);
 
-#ifdef EAM_BOXED_TEXT
-    fprintf(fp, "set style textbox");
     save_style_textbox(fp);
-#endif
 
     save_offsets(fp, "set offsets");
 
-    /* FIXME */
     fprintf(fp, "\
 set pointsize %g\n\
 set pointintervalbox %g\n\
@@ -612,6 +655,12 @@ set encoding %s\n\
 	    (polar) ? "" : "un",
 	    (parametric) ? "" : "un");
 
+    if (spiderplot) {
+	fprintf(fp, "set spiderplot\n");
+	save_style_spider(fp);
+    } else 
+	fprintf(fp, "unset spiderplot\n");
+
     if (numeric_locale)
 	fprintf(fp, "set decimalsign locale \"%s\"\n", numeric_locale);
     if (decimalsign != NULL)
@@ -625,6 +674,10 @@ set encoding %s\n\
     fputs("set view ", fp);
     if (splot_map == TRUE)
 	fprintf(fp, "map scale %g", mapview_scale);
+    else if (xz_projection)
+	fprintf(fp, "projection xz");
+    else if (yz_projection)
+	fprintf(fp, "projection yz");
     else {
 	fprintf(fp, "%g, %g, %g, %g",
 	    surface_rot_x, surface_rot_z, surface_scale, surface_zscale);
@@ -694,6 +747,7 @@ set isosamples %d, %d\n\
 	fprintf(fp, "set datafile fortran\n");
     if (df_nofpe_trap)
 	fprintf(fp, "set datafile nofpe_trap\n");
+    fprintf(fp, "set datafile %scolumnheaders\n", df_columnheaders ? "" : "no");
 
     save_hidden3doptions(fp);
     fprintf(fp, "set cntrparam order %d\n", contour_order);
@@ -777,41 +831,19 @@ set origin %g,%g\n",
     save_tics(fp, &R_AXIS);
     save_tics(fp, &THETA_AXIS);
     for (axis=0; axis<num_parallel_axes; axis++)
-	save_tics(fp, &parallel_axis[axis]);
+	save_tics(fp, &parallel_axis_array[axis]);
 
-#define SAVE_AXISLABEL_OR_TITLE(name,suffix,lab,savejust)		 \
-    {									 \
-	fprintf(fp, "set %s%s \"%s\" ",					 \
-		name, suffix, lab.text ? conv_text(lab.text) : "");	 \
-	fprintf(fp, "\nset %s%s ", name, suffix);			 \
-	save_position(fp, &(lab.offset), 3, TRUE);			 \
-	fprintf(fp, " font \"%s\"", lab.font ? conv_text(lab.font) : "");\
-	save_textcolor(fp, &(lab.textcolor));				 \
-	if (savejust && (lab.pos != CENTRE)) save_justification(lab.pos,fp); \
-	if (lab.tag == ROTATE_IN_3D_LABEL_TAG)				 \
-	    fprintf(fp, " rotate parallel");				 \
-	else if (lab.rotate == TEXT_VERTICAL)				 \
-	    fprintf(fp, " rotate");					 \
-	else if (lab.rotate)						 \
-	    fprintf(fp, " rotate by %d", lab.rotate);			 \
-	else								 \
-	    fprintf(fp, " norotate");					 \
-	fprintf(fp, "%s\n", (lab.noenhanced) ? " noenhanced" : "");	 \
-    }
+    save_axis_label_or_title(fp, "", "title", &title, TRUE);
 
-    SAVE_AXISLABEL_OR_TITLE("", "title", title, TRUE);
-
-    /* FIXME */
     fprintf(fp, "set timestamp %s \n", timelabel_bottom ? "bottom" : "top");
-    SAVE_AXISLABEL_OR_TITLE("", "timestamp", timelabel, FALSE);
+    save_axis_label_or_title(fp, "", "timestamp", &timelabel, FALSE);
 
     save_prange(fp, axis_array + T_AXIS);
     save_prange(fp, axis_array + U_AXIS);
     save_prange(fp, axis_array + V_AXIS);
 
 #define SAVE_AXISLABEL(axis)					\
-    SAVE_AXISLABEL_OR_TITLE(axis_name(axis),"label",	\
-			    axis_array[axis].label, TRUE)
+    save_axis_label_or_title(fp, axis_name(axis), "label", &axis_array[axis].label, TRUE)
 
     SAVE_AXISLABEL(FIRST_X_AXIS);
     SAVE_AXISLABEL(SECOND_X_AXIS);
@@ -832,11 +864,21 @@ set origin %g,%g\n",
     SAVE_AXISLABEL(POLAR_AXIS);
     save_prange(fp, axis_array + POLAR_AXIS);
 
-    for (axis=0; axis<num_parallel_axes; axis++)
-	save_prange(fp, &parallel_axis[axis]);
+    for (axis=0; axis<num_parallel_axes; axis++) {
+	struct axis *paxis = &parallel_axis_array[axis];
+	save_prange(fp, paxis);
+	if (paxis->label.text)
+	    save_axis_label_or_title(fp,
+				axis_name(paxis->index),"label",
+				&paxis->label, TRUE);
+	if (paxis->zeroaxis) {
+	    fprintf(fp, "set paxis %d", axis+1);
+	    save_linetype(fp, paxis->zeroaxis, FALSE);
+	    fprintf(fp, "\n");
+	}
+    }
 
 #undef SAVE_AXISLABEL
-#undef SAVE_AXISLABEL_OR_TITLE
 
     fputs("unset logscale\n", fp);
     for (axis = 0; axis < NUMBER_OF_MAIN_VISIBLE_AXES; axis++) {
@@ -907,16 +949,16 @@ set origin %g,%g\n",
     }
     fputs("\n", fp);
 
-#if (0)
     fprintf(fp, "set pm3d %s %s\n",
-		pm3d.clip == PM3D_CLIP_1IN ? "clip1in" : "clip4in",
+		pm3d.clip == PM3D_CLIP_1IN ? "clip1in" : 
+		pm3d.clip == PM3D_CLIP_4IN ? "clip4in" : "clip z",
 		pm3d.no_clipcb ? "noclipcb" : "");
-#endif
 
     if (pm3d_shade.strength <= 0)
 	fputs("set pm3d nolighting\n",fp);
     else
-	fprintf(fp, "set pm3d lighting primary %g specular %g\n", pm3d_shade.strength, pm3d_shade.spec);
+	fprintf(fp, "set pm3d lighting primary %g specular %g spec2 %g\n",
+		pm3d_shade.strength, pm3d_shade.spec, pm3d_shade.spec2);
 
     /*
      *  Save palette information
@@ -1009,13 +1051,10 @@ set origin %g,%g\n",
 	fputc('\n', fp);
     }
 
-    fputs("set fontpath ", fp);
-    {
-	char *s;
-	while ((s = save_fontpath()) != NULL)
-	    fprintf(fp, "\"%s\" ", s);
-	fputc('\n', fp);
-    }
+    if (PS_fontpath)
+	fprintf(fp, "set fontpath \"%s\"\n", PS_fontpath);
+    else
+	fprintf(fp, "set fontpath\n");
 
     if (PS_psdir)
 	fprintf(fp, "set psdir \"%s\"\n", PS_psdir);
@@ -1027,20 +1066,7 @@ set origin %g,%g\n",
 	fprintf(fp, " nologfile");
     else if (fitlogfile)
 	fprintf(fp, " logfile \'%s\'", fitlogfile);
-    switch (fit_verbosity) {
-	case QUIET:
-	    fprintf(fp, " quiet");
-	    break;
-	case RESULTS:
-	    fprintf(fp, " results");
-	    break;
-	case BRIEF:
-	    fprintf(fp, " brief");
-	    break;
-	case VERBOSE:
-	    fprintf(fp, " verbose");
-	    break;
-    }
+    fprintf(fp, " %s", reverse_table_lookup(fit_verbosity_level, fit_verbosity));
     fprintf(fp, " %serrorvariables",
 	fit_errorvariables ? "" : "no");
     fprintf(fp, " %scovariancevariables",
@@ -1062,7 +1088,7 @@ set origin %g,%g\n",
 	    fprintf(fp, " limit_abs %g", epsilon_abs);
 
 	v = get_udv_by_name((char *)FITMAXITER);
-	i = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real_int(&(v->udv_value)) : -1;
+	i = ((v != NULL) && (v->udv_value.type != NOTDEFINED)) ? real(&(v->udv_value)) : -1;
 	if (i > 0)
 	    fprintf(fp, " maxiter %i", i);
 
@@ -1239,27 +1265,43 @@ save_style_parallel(FILE *fp)
     fprintf(fp, "\n");
 }
 
-#ifdef EAM_BOXED_TEXT
+void
+save_style_spider(FILE *fp)
+{
+    fprintf(fp, "set style spiderplot ");
+    save_linetype(fp, &(spiderplot_style.lp_properties), TRUE);
+    fprintf(fp, "\nset style spiderplot fillstyle ");
+    save_fillstyle(fp, &spiderplot_style.fillstyle);
+}
+
 void
 save_style_textbox(FILE *fp)
 {
-    fprintf(fp, " %s margins %4.1f, %4.1f",
-	    textbox_opts.opaque ? "opaque": "transparent",
-	    textbox_opts.xmargin, textbox_opts.ymargin);
-    if (textbox_opts.opaque) {
-	fprintf(fp, " fc ");
-	save_pm3dcolor(fp, &(textbox_opts.fillcolor));
+    int bs;
+    for (bs = 0; bs < NUM_TEXTBOX_STYLES; bs++) {
+	textbox_style *textbox = &textbox_opts[bs];
+	if (textbox->linewidth <= 0)
+	    continue;
+	fprintf(fp, "set style textbox ");
+	if (bs > 0)
+	    fprintf(fp,"%d ", bs);
+	fprintf(fp, " %s margins %4.1f, %4.1f",
+		textbox->opaque ? "opaque": "transparent",
+		textbox->xmargin, textbox->ymargin);
+	if (textbox->opaque) {
+	    fprintf(fp, " fc ");
+	    save_pm3dcolor(fp, &(textbox->fillcolor));
+	}
+	if (textbox->noborder) {
+	    fprintf(fp, " noborder");
+	} else {
+	    fprintf(fp, " border ");
+	    save_pm3dcolor(fp, &(textbox->border_color));
+	}
+	fprintf(fp, " linewidth %4.1f", textbox->linewidth);
+	fputs("\n",fp);
     }
-    if (textbox_opts.noborder) {
-	fprintf(fp, " noborder");
-    } else {
-	fprintf(fp, " border ");
-	save_pm3dcolor(fp, &(textbox_opts.border_color));
-    }
-    fprintf(fp, " linewidth %4.1f", textbox_opts.linewidth);
-    fputs("\n",fp);
 }
-#endif
 
 void
 save_position(FILE *fp, struct position *pos, int ndim, TBOOLEAN offset)
@@ -1308,6 +1350,8 @@ save_position(FILE *fp, struct position *pos, int ndim, TBOOLEAN offset)
 void
 save_prange(FILE *fp, struct axis *this_axis)
 {
+    TBOOLEAN noextend = FALSE;
+
     fprintf(fp, "set %srange [ ", axis_name(this_axis->index));
     if (this_axis->set_autoscale & AUTOSCALE_MIN) {
 	if (this_axis->min_constraint & CONSTRAINT_LOWER ) {
@@ -1337,30 +1381,30 @@ save_prange(FILE *fp, struct axis *this_axis)
 	save_num_or_time_input(fp, this_axis->set_max, this_axis);
     }
 
-    fprintf(fp, " ] %sreverse %swriteback",
+    if (this_axis->index < PARALLEL_AXES)
+	fprintf(fp, " ] %sreverse %swriteback",
 	    ((this_axis->range_flags & RANGE_IS_REVERSED)) ? "" : "no",
 	    this_axis->range_flags & RANGE_WRITEBACK ? "" : "no");
+    else
+	fprintf(fp, " ] ");
 
-    if (this_axis->index >= PARALLEL_AXES) {
-	fprintf(fp, "\n");
-	return;
+    if ((this_axis->set_autoscale & AUTOSCALE_FIXMIN)
+    &&  (this_axis->set_autoscale & AUTOSCALE_FIXMAX)) {
+	fprintf(fp, " noextend");
+	noextend = TRUE;
     }
 
     if (this_axis->set_autoscale && fp == stderr) {
 	/* add current (hidden) range as comments */
 	fputs("  # (currently [", fp);
-	if (this_axis->set_autoscale & AUTOSCALE_MIN) {
-	    save_num_or_time_input(fp, this_axis->min, this_axis);
-	}
+	save_num_or_time_input(fp, this_axis->min, this_axis);
 	putc(':', fp);
-	if (this_axis->set_autoscale & AUTOSCALE_MAX) {
-	    save_num_or_time_input(fp, this_axis->max, this_axis);
-	}
+	save_num_or_time_input(fp, this_axis->max, this_axis);
 	fputs("] )\n", fp);
     } else
 	putc('\n', fp);
 
-    if (fp != stderr) {
+    if (!noextend && (fp != stderr)) {
 	if (this_axis->set_autoscale & (AUTOSCALE_FIXMIN))
 	    fprintf(fp, "set autoscale %sfixmin\n", axis_name(this_axis->index));
 	if (this_axis->set_autoscale & AUTOSCALE_FIXMAX)
@@ -1385,7 +1429,6 @@ save_link(FILE *fp, AXIS *this_axis)
 void
 save_nonlinear(FILE *fp, AXIS *this_axis)
 {
-#ifdef NONLINEAR_AXES
     AXIS *primary = this_axis->linked_to_primary;
 
     if (primary &&  this_axis->index == -primary->index) {
@@ -1400,7 +1443,6 @@ save_nonlinear(FILE *fp, AXIS *this_axis)
 	    fprintf(stderr, "[corrupt linkage] ");
 	fputs("\n", fp);
     }
-#endif
 }
 
 static void
@@ -1499,111 +1541,26 @@ save_pm3dcolor(FILE *fp, const struct t_colorspec *tc)
 void
 save_data_func_style(FILE *fp, const char *which, enum PLOT_STYLE style)
 {
-    switch (style) {
-    case LINES:
-	fputs("lines\n", fp);
-	break;
-    case POINTSTYLE:
-	fputs("points\n", fp);
-	break;
-    case IMPULSES:
-	fputs("impulses\n", fp);
-	break;
-    case LINESPOINTS:
-	fputs("linespoints\n", fp);
-	break;
-    case DOTS:
-	fputs("dots\n", fp);
-	break;
-    case YERRORLINES:
-	fputs("yerrorlines\n", fp);
-	break;
-    case XERRORLINES:
-	fputs("xerrorlines\n", fp);
-	break;
-    case XYERRORLINES:
-	fputs("xyerrorlines\n", fp);
-	break;
-    case YERRORBARS:
-	fputs("yerrorbars\n", fp);
-	break;
-    case XERRORBARS:
-	fputs("xerrorbars\n", fp);
-	break;
-    case XYERRORBARS:
-	fputs("xyerrorbars\n", fp);
-	break;
-    case BOXES:
-	fputs("boxes\n", fp);
-	break;
-    case HISTOGRAMS:
-	fputs("histograms\n", fp);
-	break;
-    case FILLEDCURVES:
-	fputs("filledcurves ", fp);
+    char *answer = strdup(reverse_table_lookup(plotstyle_tbl, style));
+    char *idollar = strchr(answer, '$');
+    if (idollar) {
+	do {
+	    *idollar = *(idollar+1);
+	    idollar++;
+	} while (*idollar);
+
+    }
+    fputs(answer, fp);
+    free(answer);
+
+    if (style == FILLEDCURVES) {
+	fputs(" ", fp);
 	if (!strcmp(which, "data") || !strcmp(which, "Data"))
 	    filledcurves_options_tofile(&filledcurves_opts_data, fp);
 	else
 	    filledcurves_options_tofile(&filledcurves_opts_func, fp);
-	fputc('\n', fp);
-	break;
-    case BOXERROR:
-	fputs("boxerrorbars\n", fp);
-	break;
-    case BOXXYERROR:
-	fputs("boxxyerror\n", fp);
-	break;
-    case STEPS:
-	fputs("steps\n", fp);
-	break;			/* JG */
-    case FSTEPS:
-	fputs("fsteps\n", fp);
-	break;			/* HOE */
-    case HISTEPS:
-	fputs("histeps\n", fp);
-	break;			/* CAC */
-    case VECTOR:
-	fputs("vector\n", fp);
-	break;
-    case FINANCEBARS:
-	fputs("financebars\n", fp);
-	break;
-    case CANDLESTICKS:
-	fputs("candlesticks\n", fp);
-	break;
-    case BOXPLOT:
-	fputs("boxplot\n", fp);
-	break;
-    case PM3DSURFACE:
-	fputs("pm3d\n", fp);
-	break;
-    case LABELPOINTS:
-	fputs("labels\n", fp);
-	break;
-    case IMAGE:
-	fputs("image\n", fp);
-	break;
-    case RGBIMAGE:
-	fputs("rgbimage\n", fp);
-	break;
-#ifdef EAM_OBJECTS
-	case CIRCLES:
-	fputs("circles\n", fp);
-	break;
-	case ELLIPSES:
-	fputs("ellipses\n", fp);
-	break;
-#endif
-    case SURFACEGRID:
-	fputs("surfaces\n", fp);
-	break;
-    case PARALLELPLOT:
-	fputs("parallelaxes\n", fp);
-	break;
-    case PLOT_STYLE_NONE:
-    default:
-	fputs("---error!---\n", fp);
     }
+    fputc('\n', fp);
 }
 
 void
@@ -1731,7 +1688,22 @@ save_histogram_opts (FILE *fp)
     fprintf(fp, "\n");
 }
 
-#ifdef EAM_OBJECTS
+void
+save_pixmaps(FILE *fp)
+{
+    t_pixmap *pixmap;
+    for (pixmap = pixmap_listhead; pixmap; pixmap = pixmap->next) {
+	fprintf(fp, "set pixmap %d '%s' # (%d x %d pixmap)\n",
+		pixmap->tag, pixmap->filename, pixmap->ncols, pixmap->nrows);
+	fprintf(fp, "set pixmap %d at ", pixmap->tag);
+	save_position(fp,&pixmap->pin, 3, FALSE);
+	fprintf(fp, "  size ");
+	save_position(fp,&pixmap->extent, 2, FALSE);
+	fprintf(fp, " %s %s\n",
+		pixmap->layer == LAYER_FRONT ? "front" : "behind",
+		pixmap->center ? "center" : "");
+    }
+}
 
 /* Save/show rectangle <tag> (0 means show all) */
 void
@@ -1823,7 +1795,10 @@ save_object(FILE *fp, int tag)
 	/* Properties common to all objects */
 	if (tag == 0 || tag == this_object->tag) {
 	    fprintf(fp, "\n%sobject %2d ", (fp==stderr) ? "\t" : "set ",this_object->tag);
-	    fprintf(fp, "%s ", this_object->layer > 0 ? "front" : this_object->layer < 0 ? "behind" : "back");
+	    fprintf(fp, "%s ", this_object->layer == LAYER_FRONT ? "front" :
+				this_object->layer == LAYER_DEPTHORDER ? "depthorder" :
+				this_object->layer == LAYER_BEHIND ? "behind" :
+				"back");
 	    if (this_object->clip == OBJ_NOCLIP)
 		fputs("noclip ", fp);
 	    else 
@@ -1837,7 +1812,7 @@ save_object(FILE *fp, int tag)
 	    fprintf(fp, " fc ");
 	    if (this_object->lp_properties.l_type == LT_DEFAULT)
 		    fprintf(fp,"default");
-	    else /* FIXME: Broke with removal of use_palette? */
+	    else
 		    save_pm3dcolor(fp, &this_object->lp_properties.pm3d_color);
 	    fprintf(fp, " fillstyle ");
 	    save_fillstyle(fp, &this_object->fillstyle);
@@ -1848,5 +1823,24 @@ save_object(FILE *fp, int tag)
 	int_error(c_token, "object not found");
 }
 
-#endif
+/* Save/show special polygon objects created by "set wall" */
+void
+save_walls(FILE *fp)
+{
+    static const char* wall_name[5] = {"y0", "x0", "y1", "x1", "z0"};
+    t_object *this_object;
+    int i;
+
+    for (i = 0; i < 5; i++) {
+    	this_object = &grid_wall[i];
+	if (this_object->layer == LAYER_FRONTBACK) {
+	    fprintf(fp, "set wall %s ", wall_name[i]);
+	    fprintf(fp, " fc ");
+	    save_pm3dcolor(fp, &this_object->lp_properties.pm3d_color);
+	    fprintf(fp, " fillstyle ");
+	    save_fillstyle(fp, &this_object->fillstyle);
+	}
+
+    }
+}
 
